package de.fzj.unicore.xuudb.client.functional;

import de.fzJuelich.unicore.xuudb.LoginDataType;
import de.fzj.unicore.xuudb.client.wsapi.XUUDBResponse;

public class TestServerMode extends TestFuncBase {
	private String rejectInfo = "Database mode is set to 'normal'. Only full certificates can be used.";
	private static boolean normalInit = false;

	@Override
	protected void setUp() throws Exception {
		serverConfigFile = "src/test/resources/serverNormalMode.conf";
		clientConfigFile = "src/test/resources/clientNormalMode.conf";

		if (!normalInit) {
			normalInit = true;
		}
		super.setUp();

	}

	public void testAddDNReject() {
		System.out.println("Test reject addDN user in server normal mode");
		try {
			admin.adddn(gcId, dn, xlogin2, role, project);
			fail();
		} catch (Exception e) {
			assertTrue(String.valueOf(e.getCause()).contains(rejectInfo));
		}
	}

	public void testCheckDNReject() {

		System.out.println("Test reject check-dn in server normal mode");
		try {
			admin.adddn(gcId, dn, xlogin1, role, project);
			fail();
		} catch (Exception e) {
			assertTrue(String.valueOf(e.getCause()).contains(rejectInfo));

		}

	}

	public void testAddAction() {
		System.out.println("Test add user in server normal mode");

		try {
			XUUDBResponse resp = admin.add(gcId, certPem, xlogin1, role,
					project);
			if (!resp.getStatus().contains("OK"))
				fail();
		} catch (Exception e) {
			fail(e.toString());
		}
	}

	public void testUpdateReject() {

		System.out.println("Test reject update by DN in server normal mode");

		try {
			LoginDataType lg = LoginDataType.Factory.newInstance();
			lg.setGcID(gcId + "xxx");
			lg.setRole("newRole");
			admin.update(gcId, dn, lg);
			fail();
		} catch (Exception e) {

			assertTrue(String.valueOf(e.getCause()).contains(rejectInfo));

		}
	}

	public void testListReject() {

		System.out.println("Test reject list by DN in server normal mode");

		try {
			LoginDataType lg = LoginDataType.Factory.newInstance();
			lg.setGcID(gcId);
			lg.setToken(dn);
			admin.list(lg);
			fail();
		} catch (Exception e) {

			assertTrue(String.valueOf(e.getCause()).contains(rejectInfo));

		}
	}

}
